import fitz, os
from PIL import Image
class convert_main:
    def signal_pdf_to_image(self,pdf_path: str,image_path:str,image_page:int,image_name:str,dpi:int):
        pdf_document = fitz.open(pdf_path)
        page = pdf_document.load_page(image_page)
        image = page.get_pixmap(matrix=fitz.Matrix(dpi / 72, dpi / 72))
        pil_image = Image.frombytes("RGB", [image.width, image.height], image.samples)
        image_file = os.path.join(image_path, f"{image_name}.png")
        pil_image.save(image_file, dpi=(dpi, dpi))


from typing import List, Union ,Literal
import os, fitz
from concurrent.futures import ThreadPoolExecutor
from ..information import gain
from ..path import paths
from ._convert_main import convert_main
class pdf_convert:
    def __init__(self,theard_pool:bool=False,sigle_document:bool=False,max_workers:int=4) ->None:
        self.theard_pool=theard_pool
        self.single_document=sigle_document
        self.max_workers=max_workers
    def pdf_to_picture(self,pdf_paths:Union[str, List[str]],image_path:str,dpi:int=300):
        if self.single_document:
            if not isinstance(pdf_paths, str):
                raise ValueError(f"pdf_paths:{pdf_paths}参数类型错误，请输入str类型")
            else:
                if os.path.isfile(pdf_paths):
                    paths().maker_path(image_path)
                    xian_pool = ThreadPoolExecutor(max_workers=self.max_workers)
                    if self.theard_pool:
                        for pages_number in range(fitz.open(pdf_paths).page_count):
                            xian_pool.submit(convert_main().signal_pdf_to_image,pdf_paths,
                                             image_path,pages_number,str(pages_number+1),dpi)
                        xian_pool.shutdown()
                    else:
                        for pages_number in range(fitz.open(pdf_paths).page_count):
                            convert_main().signal_pdf_to_image(pdf_paths, image_path, pages_number,
                                             str(pages_number + 1), dpi)
                else:
                    raise FileExistsError(f"pdf_paths:{pdf_paths}文件不存在")
        else:
            if not isinstance(pdf_paths, list) or not all(isinstance(path, str) for path in pdf_paths):
                raise ValueError(f"pdf_paths:{pdf_paths}参数类型错误，请输入list[str]类型")
            else:
                if not all(os.path.isfile(path) for path in pdf_paths):
                    raise FileExistsError(f"pdf_paths:{pdf_paths}文件缺失")
                else:
                    paths().maker_path(image_path)
                    image_name_list=list()
                    for image_name in range(len(pdf_paths)):
                        image_name_list.append(os.path.join(image_path,gain().get_random_file_name(12)))
                    for image_name in image_name_list:
                        paths().maker_path(image_name)
                    xian_pool=ThreadPoolExecutor(max_workers=self.max_workers)
                    if self.theard_pool:
                        for pdf_count in range(len(pdf_paths)):
                            for pages_number in range(fitz.open(pdf_paths[pdf_count]).page_count):
                                xian_pool.submit(convert_main().signal_pdf_to_image,pdf_paths[pdf_count],
                                                 image_name_list[pdf_count],pages_number,str(pages_number+1),dpi)
                        xian_pool.shutdown()
                    else:
                        paths().maker_path(image_path)
                        image_name_list = list()
                        for image_name in range(len(pdf_paths)):
                            image_name_list.append(os.path.join(image_path, gain().get_random_file_name(12)))
                        for image_name in image_name_list:
                            paths().maker_path(image_name)
                        for pdf_count in range(len(pdf_paths)):
                            for pages_number in range(fitz.open(pdf_paths[pdf_count]).page_count):
                                convert_main().signal_pdf_to_image(pdf_paths[pdf_count],image_name_list[pdf_count],
                                                                   pages_number,str(pages_number+1),dpi)




import os


class paths:
    def get_document_names(self,path:str,endswith:str='',path_or_name:bool=True,get_folder:bool=False)->list:
        try:
            path = os.path.abspath(path)
            if not os.path.exists(path):
                raise FileNotFoundError(f"path:{path}不存在")
            else:
                path_names = os.listdir(path)
                if path_or_name==True:
                    for i in range(len(path_names)):
                        path_names[i] = os.path.join(path, path_names[i])
                new_path_names = list()
                dict=[".pdf",'.png','.jpg','.jpeg','.txt','.doc',
                      '.docx','.gif','.mp4','.mp3','.avi','.wav',
                      '.xls','.xlsx','.ppt','.pptx','.zip','.exe'
                      ,'.rar','.mov','.bak','.htm','.html']
                if get_folder== True and endswith=='':
                    new_path_names=path_names
                    return new_path_names
                elif get_folder==False and endswith!='' and endswith.lower() in dict:
                    for name in path_names:
                        if name.endswith(endswith):
                            new_path_names.append(name)
                    return new_path_names
                else:
                    raise ValueError(f"endwith:{endswith}和get_folder:{get_folder}参数错误")
        except:
            raise ValueError(f"path:{path}参数错误")

    def maker_path(self,path:str)->None:
        try:
            if not os.path.exists(path):
                os.mkdir(path)
        except:
            raise ValueError(f"path:{path}参数错误")



from . import (
    convert,
    path,
    information,
)

from .convert import (
    pdf_convert
)

from .path import (
    paths
)

from .information import (
    gain
)

__all__ = [
    "__version__",
    "convert",
    "path",
    "information",
    "pdf_convert",
    "paths",
    "gain"
]

__version__ = '0.0.1'
__description__ = 'Consolidation package for daily use'
__author__ = 'S Liao'


import random, secrets
class gain:
    def get_random_file_name(self,length_character: int)->str:
        if length_character <= 0:
            raise ValueError(f"length_character:{length_character}必须大于0")
        alphabet = 'abcdefghijklmnopqrstuvwxyz'
        if length_character > len(alphabet):
            return ''.join(secrets.choice(alphabet) for _ in range(length_character))
        else:
            return ''.join(random.sample(alphabet, length_character))


import codecs
import os
from setuptools import setup, find_packages

# these things are needed for the README.md show on pypi (if you dont need delete it)
here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

# you need to change all these
VERSION = '0.0.8'
DESCRIPTION = 'Consolidation package for daily use'

setup(
    name="gqxls",
    version=VERSION,
    author="S Liao",
    author_email="shunliaopetroleum@163.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    license="GNU General Public License v3.0",
    project_urls={
        'Source': 'https://github.com/891011well/gqxls',
        "Bug Tracker": "https://github.com/891011well/gqxls/issues",
    },
    packages=find_packages(),
    install_requires=['pillow','pymupdf'],
    keywords=['python', 'gqxls'],
    classifiers=[  # see https://pypi.org/classifiers/
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        "Programming Language :: Python :: 3.11",
    ],
    platforms='any',
    python_requires='>=3.6',
)
